@@ -5,7 +5,8 @@
 # param_optim_lower - lower bound of params
 # param_optim_upper - upper
 # param_optim_start - current param values
-# param_optim_start0 - some central param values that can be used for optimization restarts
+# param_optim_start0 - some central param values that can be used for
+#  optimization restarts
 # param_optim_jitter - how to jitter params in optimization
 
 # Suggested
@@ -16,9 +17,9 @@
 
 
 
-#' Periodic Kernel R6 class
+#' Ordered Factor Kernel R6 class
 #'
-#' p is the period for each dimension, a is a single number for scaling
+#' Use for factor inputs that are considered to have an ordering
 #'
 #' @docType class
 #' @importFrom R6 R6Class
@@ -31,8 +32,8 @@
 #' @format \code{\link{R6Class}} object.
 #' @field p Parameter for correlation
 #' @field p_est Should p be estimated?
-#' @field p_lower Lower bound of logp
-#' @field p_upper Upper bound of logp
+#' @field p_lower Lower bound of p
+#' @field p_upper Upper bound of p
 #' @field p_length length of p
 #' @field s2 variance
 #' @field s2_est Is s2 estimated?
@@ -41,6 +42,9 @@
 #' @field logs2_upper Upper bound of logs2
 #' @field xindex Index of the factor (which column of X)
 #' @field nlevels Number of levels for the factor
+#' @field offdiagequal What should offdiagonal values be set to when the
+#' indices are the same? Use to avoid decomposition errors, similar to
+#' adding a nugget.
 #' @examples
 #' kk <- OrderedFactorKernel$new(D=1, nlevels=5, xindex=1)
 #' kk$p <- (1:10)/100
@@ -84,9 +88,8 @@ OrderedFactorKernel <- R6::R6Class(
   classname = "GauPro_kernel_OrderedFactorKernel",
   inherit = GauPro_kernel,
   public = list(
-    p = NULL, # Period
+    p = NULL,
     p_est = NULL,
-    # logp = NULL,
     p_lower = NULL,
     p_upper = NULL,
     p_length = NULL,
@@ -97,13 +100,9 @@ OrderedFactorKernel <- R6::R6Class(
     logs2_upper = NULL,
     nlevels = NULL,
     xindex = NULL,
-    # alpha = NULL,
-    # logalpha = NULL,
-    # logalpha_lower = NULL,
-    # logalpha_upper = NULL,
-    # alpha_est = NULL,
+    offdiagequal = NULL,
     #' @description Initialize kernel object
-    #' @param p Periodic parameter
+    #' @param p Vector of distances in latent space
     #' @param s2 Initial variance
     #' @param D Number of input dimensions of data
     #' @param p_lower Lower bound for p
@@ -114,13 +113,23 @@ OrderedFactorKernel <- R6::R6Class(
     #' @param s2_est Should s2 be estimated?
     #' @param xindex Index of X to use the kernel on
     #' @param nlevels Number of levels for the factor
-    initialize = function(s2=1, D, nlevels, xindex,
-                          p_lower=0, p_upper=1, p_est=TRUE,
-                          s2_lower=1e-8, s2_upper=1e8, s2_est=TRUE
+    #' @param useC Should C code used? Much faster.
+    #' @param offdiagequal What should offdiagonal values be set to when the
+    #' indices are the same? Use to avoid decomposition errors, similar to
+    #' adding a nugget.
+    initialize = function(s2=1, D=NULL, nlevels, xindex,
+                          p_lower=1e-8, p_upper=5, p_est=TRUE,
+                          s2_lower=1e-8, s2_upper=1e8, s2_est=TRUE,
+                          useC=TRUE, offdiagequal=1-1e-6
     ) {
-      # Must give in D
-      if (missing(D)) {stop("Must give Index kernel D")}
+      # Don't require giving in D since it doesn't matter
 
+      # stopifnot(is.numeric(D), length(D)==1, D>=1, abs(D-round(D))<1e-16)
+      stopifnot(is.numeric(nlevels), length(nlevels)==1, nlevels>=2,
+                abs(nlevels-round(nlevels))<1e-16)
+      stopifnot(is.numeric(xindex), length(xindex)==1, xindex>=1,
+                abs(xindex-round(xindex))<1e-16)
+      # stopifnot(xindex <= D)
       self$D <- D
       self$nlevels <- nlevels
       self$xindex <- xindex
@@ -129,24 +138,14 @@ OrderedFactorKernel <- R6::R6Class(
       p <- rep(1, nlevels - 1)
       self$p <- p
       self$p_length <- length(p)
-      # Ensure separation between levels to avoid instability
-      self$p_lower <- rep(1e-1, self$p_length)
-      # Don't give upper 1 since it will give optimization error
-      self$p_upper <- rep(5, self$p_length)
-      # self$logp <- log(p, 10)
 
-      # Now set upper and lower so they have correct length
-      # self$logp_lower <- log(p_lower, 10)
-      # self$logp_upper <- log(p_upper, 10)
-      # Setting logp_lower so dimensions are right
-      # logp_lower <- log(p_lower, 10)
-      # logp_upper <- log(p_upper, 10)
-      # self$logp_lower <- if (length(logp_lower) == self$p_length) {logp_lower}
-      # else if (length(logp_lower)==1) {rep(logp_lower, self$p_length)}
-      # else {stop("Error for kernel_Periodic logp_lower")}
-      # self$logp_upper <- if (length(logp_upper) == self$p_length) {logp_upper}
-      # else if (length(logp_upper)==1) {rep(logp_upper, self$p_length)}
-      # else {stop("Error for kernel_Periodic logp_upper")}
+      stopifnot(is.numeric(p_lower), length(p_lower)==1, p_lower>=0)
+      stopifnot(is.numeric(p_upper), length(p_upper)==1, p_upper>=0)
+      stopifnot(p_lower <= p_upper)
+      # Ensure separation between levels to avoid instability
+      self$p_lower <- rep(p_lower, self$p_length)
+      # Don't give upper 1 since it will give optimization error
+      self$p_upper <- rep(p_upper, self$p_length)
 
       self$p_est <- p_est
       self$s2 <- s2
@@ -154,7 +153,9 @@ OrderedFactorKernel <- R6::R6Class(
       self$logs2_lower <- log(s2_lower, 10)
       self$logs2_upper <- log(s2_upper, 10)
       self$s2_est <- s2_est
+      self$useC <- useC
 
+      self$offdiagequal <- offdiagequal
     },
     #' @description Calculate covariance between two points
     #' @param x vector.
@@ -166,20 +167,12 @@ OrderedFactorKernel <- R6::R6Class(
     k = function(x, y=NULL, p=self$p, s2=self$s2, params=NULL) {
       if (!is.null(params)) {
         lenparams <- length(params)
-        # logp <- params[1:(lenpar-2)]
-        # logalpha <- params[lenpar-1]
-        # logs2 <- params[lenpar]
 
         if (self$p_est) {
           p <- params[1:self$p_length]
         } else {
           p <- self$p
         }
-        # if (self$alpha_est) {
-        #   logalpha <- params[1 + as.integer(self$p_est) * self$p_length]
-        # } else {
-        #   logalpha <- self$logalpha
-        # }
         if (self$s2_est) {
           logs2 <- params[lenparams]
         } else {
@@ -192,29 +185,33 @@ OrderedFactorKernel <- R6::R6Class(
         s2 <- 10^logs2
       } else {
         if (is.null(p)) {p <- self$p}
-        # if (is.null(logalpha)) {logalpha <- self$logalpha}
         if (is.null(s2)) {s2 <- self$s2}
       }
-      # p <- 10^logp
-      # alpha <- 10^logalpha
       if (is.null(y)) {
         if (is.matrix(x)) {
-          # val <- outer(1:nrow(x), 1:nrow(x),
-          #              Vectorize(function(i,j){
-          #                self$kone(x[i,],x[j,],p=p, s2=s2)
-          #              }))
-          val <- outer(1:nrow(x), 1:nrow(x),
-                       Vectorize(function(i,j){
-                         self$kone(x[i,],x[j,],p=p, s2=s2, isdiag=i==j)
-                       }))
-          # if (inherits(cgmtry,"try-error")) {browser()}
+          if (self$useC) {
+            val <- s2 * corr_orderedfactor_matrix_symC(x, p, self$xindex,
+                                                       self$offdiagequal)
+          } else {
+            val <- outer(1:nrow(x), 1:nrow(x),
+                         Vectorize(function(i,j){
+                           self$kone(x[i,],x[j,],p=p, s2=s2, isdiag=i==j)
+                         }))
+          }
           return(val)
         } else {
           return(s2 * 1)
         }
       }
       if (is.matrix(x) & is.matrix(y)) {
-        outer(1:nrow(x), 1:nrow(y), Vectorize(function(i,j){self$kone(x[i,],y[j,],p=p, s2=s2)}))
+        if (self$useC) { # Way faster
+          s2 * corr_orderedfactor_matrixmatrixC(
+            x=x, y=y, theta=p, xindex=self$xindex,
+            offdiagequal=self$offdiagequal)
+        } else {
+          outer(1:nrow(x), 1:nrow(y),
+                Vectorize(function(i,j){self$kone(x[i,],y[j,],p=p, s2=s2)}))
+        }
       } else if (is.matrix(x) & !is.matrix(y)) {
         apply(x, 1, function(xx) {self$kone(xx, y, p=p, s2=s2)})
       } else if (is.matrix(y)) {
@@ -232,10 +229,9 @@ OrderedFactorKernel <- R6::R6Class(
     #' @param offdiagequal What should offdiagonal values be set to when the
     #' indices are the same? Use to avoid decomposition errors, similar to
     #' adding a nugget.
-    #' @references https://stackoverflow.com/questions/27086195/linear-index-upper-triangular-matrix
-    kone = function(x, y, p, s2, isdiag=TRUE, offdiagequal=1-1e-6) {
-      # if (missing(p)) {p <- 10^logp}
-      # out <- s2 * exp(-sum(alpha*sin(p * (x-y))^2))
+    #' @references
+    #' https://stackoverflow.com/questions/27086195/linear-index-upper-triangular-matrix
+    kone = function(x, y, p, s2, isdiag=TRUE, offdiagequal=self$offdiagequal) {
       x <- x[self$xindex]
       y <- y[self$xindex]
       stopifnot(x>=1, y>=1, x<=self$nlevels, y<=self$nlevels,
@@ -266,8 +262,7 @@ OrderedFactorKernel <- R6::R6Class(
     #' @param C_nonug Covariance without nugget added to diagonal
     #' @param C Covariance with nugget
     #' @param nug Value of nugget
-    dC_dparams = function(params=NULL, X, C_nonug, C, nug) {#browser(text = "Make sure all in one list")
-      # stop("not implemented, kernel index, dC_dp")
+    dC_dparams = function(params=NULL, X, C_nonug, C, nug) {
       n <- nrow(X)
 
       lenparams <- length(params)
@@ -278,11 +273,6 @@ OrderedFactorKernel <- R6::R6Class(
         } else {
           p <- self$p
         }
-        # if (self$alpha_est) {
-        #   logalpha <- params[1 + as.integer(self$p_est) * self$p_length]
-        # } else {
-        #   logalpha <- self$logalpha
-        # }
         if (self$s2_est) {
           logs2 <- params[lenparams]
         } else {
@@ -290,80 +280,68 @@ OrderedFactorKernel <- R6::R6Class(
         }
       } else {
         p <- self$p
-        # logalpha <- self$logalpha
         logs2 <- self$logs2
       }
-
-      # lenparams <- length(params)
-      # logp <- params[1:(lenparams - 2)]
-      # p <- 10^logp
-      # logalpha <- params[lenparams-1]
-      # alpha <- 10^logalpha
       log10 <- log(10)
-      # logs2 <- params[lenparams]
       s2 <- 10 ^ logs2
 
-      # if (is.null(params)) {params <- c(self$logp, self$logalpha, self$logs2)}
       if (missing(C_nonug)) { # Assume C missing too, must have nug
         C_nonug <- self$k(x=X, params=params)
         C <- C_nonug + diag(nug*s2, nrow(C_nonug))
       }
 
       lenparams_D <- self$p_length*self$p_est + self$s2_est
-      dC_dparams <- array(dim=c(lenparams_D, n, n), data=0)
-      if (self$s2_est) {
-        dC_dparams[lenparams_D,,] <- C * log10
-      }
-      if (self$p_est) {
-        for (k in 1:length(p)) { # k is index of parameter
-          for (i in seq(1, n-1, 1)) {
-            for (j in seq(i+1, n, 1)) {
-              xx <- X[i, self$xindex]
-              yy <- X[j, self$xindex]
-              if (xx == yy) {
-                # Corr is just 1, parameter has no effect
-              } else {
-                # ii <- min(xx-1, yy-1)
-                # jj <- max(xx-1, yy-1)
-                # nn <- self$nlevels
-                # ind <- (nn*(nn-1)/2) - (nn-ii)*((nn-ii)-1)/2 + jj - ii #- 1
-                ii <- min(xx,yy)
-                jj <- max(xx,yy) - 1
-                if (ii <= k && k <= jj) { # Does correspond to the correct parameter
-                  p_dist <- sum(p[ii:jj])
-                  r <- exp(-p_dist^2)
-                  dC_dparams[k,i,j] <- -2 * p_dist * r * s2
-                  dC_dparams[k,j,i] <- dC_dparams[k,i,j]
 
+
+      if (self$useC) {
+        dC_dparams <- kernel_orderedFactor_dC(X, p, C_nonug, self$s2_est,
+                                              self$p_est, lenparams_D, s2*nug,
+                                              self$xindex-1,
+                                              self$nlevels, s2)
+        if (any(is.na(dC_dparams))) {
+          print('bad dcdparams')
+          browser('fix this')
+          warning('a;slkdfja;sdlkfj')
+        }
+      } else {
+        dC_dparams <- array(dim=c(lenparams_D, n, n), data=0)
+        if (self$s2_est) {
+          dC_dparams[lenparams_D,,] <- C * log10
+        }
+        if (self$p_est) {
+          for (k in 1:length(p)) { # k is index of parameter
+            for (i in seq(1, n-1, 1)) {
+              for (j in seq(i+1, n, 1)) {
+                xx <- X[i, self$xindex]
+                yy <- X[j, self$xindex]
+                if (xx == yy) {
+                  # Corr is just 1, parameter has no effect
                 } else {
-                  # Parameter has no effect
+                  # ii <- min(xx-1, yy-1)
+                  # jj <- max(xx-1, yy-1)
+                  # nn <- self$nlevels
+                  # ind <- (nn*(nn-1)/2) - (nn-ii)*((nn-ii)-1)/2 + jj - ii #- 1
+                  ii <- min(xx,yy)
+                  jj <- max(xx,yy) - 1
+                  if (ii <= k && k <= jj) {
+                    # Does correspond to the correct parameter
+                    p_dist <- sum(p[ii:jj])
+                    r <- exp(-p_dist^2)
+                    dC_dparams[k,i,j] <- -2 * p_dist * r * s2
+                    dC_dparams[k,j,i] <- dC_dparams[k,i,j]
+
+                  } else {
+                    # Parameter has no effect
+                  }
                 }
               }
-              #
-              # r2 <- sum(p * (X[i,]-X[j,])^2)
-              # dC_dparams[k,i,j] <- -C_nonug[i,j] * alpha * sin(2*p[k]*(X[i,k] - X[j,k])) * (X[i,k] - X[j,k]) * p[k] * log10
-              # dC_dparams[k,j,i] <- dC_dparams[k,i,j]
             }
-          }
-          for (i in seq(1, n, 1)) { # Get diagonal set to zero
-            dC_dparams[k,i,i] <- 0
+            for (i in seq(1, n, 1)) { # Get diagonal set to zero
+              dC_dparams[k,i,i] <- 0
+            }
           }
         }
       }
-      # # Grad for logalpha
-      # if (self$alpha_est) {
-      #   alph_ind <- lenparams_D - as.integer(self$s2_est)
-      #   for (i in seq(1, n-1, 1)) {
-      #     for (j in seq(i+1, n, 1)) {
-      #       r2 <- -sum(sin(p * (X[i,]-X[j,]))^2)
-      #       dC_dparams[alph_ind, i,j] <- C_nonug[i,j] * r2 * alpha * log10
-      #       dC_dparams[alph_ind, j,i] <- dC_dparams[alph_ind, i,j]
-      #     }
-      #   }
-      #   for (i in seq(1, n, 1)) {
-      #     dC_dparams[alph_ind, i,i] <- 0
-      #   }
-      # }
       return(dC_dparams)
     },
     #' @description Calculate covariance matrix and its derivative
@@ -375,117 +353,89 @@ OrderedFactorKernel <- R6::R6Class(
       s2 <- self$s2_from_params(params)
       C_nonug <- self$k(x=X, params=params)
       C <- C_nonug + diag(s2*nug, nrow(X))
-      dC_dparams <- self$dC_dparams(params=params, X=X, C_nonug=C_nonug, C=C, nug=nug)
+      dC_dparams <- self$dC_dparams(params=params, X=X, C_nonug=C_nonug, C=C,
+                                    nug=nug)
       list(C=C, dC_dparams=dC_dparams)
     },
     #' @description Derivative of covariance with respect to X
     #' @param XX matrix of points
     #' @param X matrix of points to take derivative with respect to
-    #' @param logp log of p
-    #' @param logalpha log of alpha
-    #' @param s2 Variance parameter
-    dC_dx = function(XX, X, logp=self$logp, logalpha=self$logalpha, s2=self$s2) {
-      stop("not implemented, ordered factor kernel, dC_dx")
-      # if (missing(theta)) {theta <- 10^beta}
-      p <- 10 ^ logp
-      alpha <- 10 ^ logalpha
+    #' @param ... Additional args, not used
+    dC_dx = function(XX, X, ...) {
       if (!is.matrix(XX)) {stop()}
       d <- ncol(XX)
       if (ncol(X) != d) {stop()}
       n <- nrow(X)
       nn <- nrow(XX)
-      dC_dx <- array(NA, dim=c(nn, d, n))
-      for (i in 1:nn) {
-        for (j in 1:d) {
-          for (k in 1:n) {
-            # r <- sqrt(sum(theta * (XX[i,] - X[k,]) ^ 2))
-            CC <- s2 * exp(-sum(alpha * sin(p * (XX[i, ]-X[k, ]))^2))
-            dC_dx[i, j, k] <- CC * (-alpha) * sin(2*p[j]*(XX[i, j]-X[k, j])) * p[j] #* (XX[i, j] - X[k, j])
-          }
-        }
-      }
+      dC_dx <- array(0, dim=c(nn, d, n))
+      dC_dx[, self$xindex, ] <- NA
       dC_dx
     },
     #' @description Starting point for parameters for optimization
     #' @param jitter Should there be a jitter?
     #' @param y Output
     #' @param p_est Is p being estimated?
-    #' @param alpha_est Is alpha being estimated?
     #' @param s2_est Is s2 being estimated?
     param_optim_start = function(jitter=F, y, p_est=self$p_est,
                                  s2_est=self$s2_est) {
-      # Use current values for theta, partial MLE for s2
-      # vec <- c(log(self$theta, 10), log(sum((y - mu) * solve(R, y - mu)) / n), 10)
-      if (p_est) {vec <- c(self$p)} else {vec <- c()}
-      # if (alpha_est) {vec <- c(vec, self$logalpha)} else {}
-      if (s2_est) {vec <- c(vec, self$logs2)} else {}
-      # if (jitter && p_est) {
-      #   # vec <- vec + c(self$logp_optim_jitter,  0)
-      #   vec[1:length(self$p)] = vec[1:length(self$p)] + rnorm(length(self$p), 0, 1)
-      # }
+      if (p_est) {
+        vec <- pmin(pmax(self$p + jitter*rnorm(length(self$p), 0, .5),
+                         self$p_lower), self$p_upper)
+      } else {
+        vec <- c()
+      }
+      if (s2_est) {
+        vec <- c(vec, self$logs2 + jitter*rnorm(1))
+      }
       vec
     },
     #' @description Starting point for parameters for optimization
     #' @param jitter Should there be a jitter?
     #' @param y Output
     #' @param p_est Is p being estimated?
-    #' @param alpha_est Is alpha being estimated?
     #' @param s2_est Is s2 being estimated?
     param_optim_start0 = function(jitter=F, y, p_est=self$p_est,
                                   s2_est=self$s2_est) {
-      # Use 0 for theta, partial MLE for s2
-      # vec <- c(rep(0, length(self$theta)), log(sum((y - mu) * solve(R, y - mu)) / n), 10)
-      if (p_est) {vec <- rep(0, self$p_length)} else {vec <- c()}
-      # if (alpha_est) {vec <- c(vec, 1)} else {}
-      if (s2_est) {vec <- c(vec, 0)} else {}
-      if (jitter && p_est) {
-        vec[1:length(self$p)] = vec[1:length(self$logp)] + rnorm(length(self$p), 0, 1)
+      if (p_est) {
+        vec <- pmin(pmax(rep(1, length(self$p)) +
+                           jitter*rnorm(length(self$p), 0, .5),
+                         self$p_lower), self$p_upper)
+      } else {
+        vec <- c()
+      }
+      if (s2_est) {
+        vec <- c(vec, self$logs2 + jitter*rnorm(1))
       }
       vec
     },
     #' @description Lower bounds of parameters for optimization
     #' @param p_est Is p being estimated?
-    #' @param alpha_est Is alpha being estimated?
     #' @param s2_est Is s2 being estimated?
     param_optim_lower = function(p_est=self$p_est,
-                                 # alpha_est=self$alpha_est,
                                  s2_est=self$s2_est) {
-      # c(self$logp_lower, self$logs2_lower)
       if (p_est) {vec <- c(self$p_lower)} else {vec <- c()}
-      # if (alpha_est) {vec <- c(vec, self$logalpha_lower)} else {}
       if (s2_est) {vec <- c(vec, self$logs2_lower)} else {}
       vec
     },
     #' @description Upper bounds of parameters for optimization
     #' @param p_est Is p being estimated?
-    #' @param alpha_est Is alpha being estimated?
     #' @param s2_est Is s2 being estimated?
     param_optim_upper = function(p_est=self$p_est,
-                                 # alpha_est=self$alpha_est,
                                  s2_est=self$s2_est) {
-      # c(self$logp_upper, self$logs2_upper)
       if (p_est) {vec <- c(self$p_upper)} else {vec <- c()}
-      # if (alpha_est) {vec <- c(vec, self$logalpha_upper)} else {}
       if (s2_est) {vec <- c(vec, self$logs2_upper)} else {}
       vec
     },
     #' @description Set parameters from optimization output
     #' @param optim_out Output from optimization
     #' @param p_est Is p being estimated?
-    #' @param alpha_est Is alpha being estimated?
     #' @param s2_est Is s2 being estimated?
     set_params_from_optim = function(optim_out, p_est=self$p_est,
-                                     # alpha_est=self$alpha_est,
                                      s2_est=self$s2_est) {
       loo <- length(optim_out)
       if (p_est) {
         self$p <- optim_out[1:(self$p_length)]
-        # self$p <- 10 ^ self$logp
       }
-      # if (alpha_est) {
-      #   self$logalpha <- optim_out[(1 + p_est * self$p_length)]
-      #   self$alpha <- 10 ^ self$logalpha
-      # }
       if (s2_est) {
         self$logs2 <- optim_out[loo]
         self$s2 <- 10 ^ self$logs2
@@ -502,25 +452,16 @@ OrderedFactorKernel <- R6::R6Class(
         self$s2
       }
     },
-    #' @param ... Not used.
-    plot = function(...) {
-      x1 <- 1:self$nlevels
-      X1 <- X2 <- matrix(data=0, ncol=self$D, nrow=self$nlevels)
-      X1[, self$xindex] <- x1
-      X2[, self$xindex] <- x1
-      k <- self$k(X1, X2)
-
-      df <- NULL
-      for (i in 1:self$nlevels) {
-        for (j in 1:self$nlevels) {
-          df <- rbind(df,
-                      data.frame(x1=i, x2=j, k=k[i,j]))
-        }
-      }
-      ggplot2::ggplot(data=df, ggplot2::aes(x1, x2, fill=k)) +
-        ggplot2::geom_tile() +
-        ggplot2::scale_fill_gradient(low='white', high='red', limits=c(0, NA)) +
-        ggplot2::scale_y_reverse()
+    #' @description Plot the points in the latent space
+    plotLatent = function() {
+      x <- c(0, cumsum(self$p))
+      pdf <- data.frame(x=x, y=0)
+      pdf$name <- paste0("x=",1:nrow(pdf))
+      ggplot2::ggplot(pdf, ggplot2::aes(x, 0, label=name)) +
+        ggplot2::geom_point() +
+        ggrepel::geom_label_repel() +
+        ggplot2::scale_y_continuous(breaks=NULL) +
+        ggplot2::ylab(NULL)
     },
     #' @description Print this object
     print = function() {
