@@ -19,16 +19,19 @@
 #' @useDynLib GauPro, .registration = TRUE
 #' @importFrom Rcpp evalCpp
 #' @importFrom stats optim
-#' @keywords data, kriging, Gaussian process, regression
+# @keywords data, kriging, Gaussian process, regression
 #' @return Object of \code{\link{R6Class}} with methods for fitting GP model.
 #' @format \code{\link{R6Class}} object.
+#' @field D Number of input dimensions of data
 #' @examples
 #' #k <- GauPro_trend$new()
-GauPro_trend <- R6::R6Class(classname = "GauPro_trend",
-                             public = list(
-                               # Gradient is -2 * t(yminusmu) %*% Siginv %*% du/db
-                             ),
-                             private = list(
+GauPro_trend <- R6::R6Class(
+  classname = "GauPro_trend",
+  public = list(
+    # Gradient is -2 * t(yminusmu) %*% Siginv %*% du/db
+    D = NULL
+  ),
+  private = list(
 
-                             )
+  )
 )
